@@ -147,10 +147,10 @@ public class SmileFactory extends JsonFactory
     public Version version() {
         return PackageVersion.VERSION;
     }
-    
+
     /*
     /**********************************************************
-    /* Format detection functionality (since 1.8)
+    /* Format detection functionality
     /**********************************************************
      */
     
@@ -161,7 +161,7 @@ public class SmileFactory extends JsonFactory
 
     // Defaults work fine for this:
     // public boolean canUseSchema(FormatSchema schema) { }
-
+    
     /**
      * Sub-classes need to override this method (as of 1.8)
      */
@@ -169,7 +169,18 @@ public class SmileFactory extends JsonFactory
     public MatchStrength hasFormat(InputAccessor acc) throws IOException {
         return SmileParserBootstrapper.hasSmileFormat(acc);
     }
+
+    /*
+    /**********************************************************
+    /* Capability introspection
+    /**********************************************************
+     */
     
+    @Override
+    public boolean canHandleBinaryNatively() {
+        return true;
+    }
+
     /*
     /**********************************************************
     /* Configuration, parser settings
@@ -264,120 +275,52 @@ public class SmileFactory extends JsonFactory
     
     /*
     /**********************************************************
-    /* Overridden parser factory methods, new (2.1)
+    /* Overridden parser factory methods: only override methods
+    /* that can use co-variance (to return SmileParser)
     /**********************************************************
      */
 
+    @SuppressWarnings("resource")
     @Override
-    public SmileParser createParser(File f)
-        throws IOException, JsonParseException
-    {
-        return _createParser(new FileInputStream(f), _createContext(f, true));
+    public SmileParser createParser(File f) throws IOException {
+        IOContext ctxt = _createContext(f, true);
+        return _createParser(_decorate(new FileInputStream(f), ctxt), ctxt);
     }
 
     @Override
-    public SmileParser createParser(URL url)
-        throws IOException, JsonParseException
-    {
-        return _createParser(_optimizedStreamFromURL(url), _createContext(url, true));
+    public SmileParser createParser(URL url) throws IOException {
+        IOContext ctxt = _createContext(url, true);
+        return _createParser(_decorate(_optimizedStreamFromURL(url), ctxt), ctxt);
     }
 
     @Override
-    public SmileParser createParser(InputStream in)
-        throws IOException, JsonParseException
-    {
-        return _createParser(in, _createContext(in, false));
+    public SmileParser createParser(InputStream in) throws IOException {
+        IOContext ctxt = _createContext(in, false);
+        return _createParser(_decorate(in, ctxt), ctxt);
     }
 
-    //public JsonParser createJsonParser(Reader r)
+    @Override
+    public SmileParser createParser(byte[] data) throws IOException {
+        return createParser(data, 0, data.length);
+    }
     
+    @SuppressWarnings("resource")
     @Override
-    public SmileParser createParser(byte[] data)
-        throws IOException, JsonParseException
-    {
+    public SmileParser createParser(byte[] data, int offset, int len) throws IOException {
         IOContext ctxt = _createContext(data, true);
-        return _createParser(data, 0, data.length, ctxt);
-    }
-    
-    @Override
-    public SmileParser createParser(byte[] data, int offset, int len)
-        throws IOException, JsonParseException
-    {
-        return _createParser(data, offset, len, _createContext(data, true));
-    }
-   
-    /*
-    /**********************************************************
-    /* Overridden parser factory methods, old (pre-2.1)
-    /**********************************************************
-     */
-    
-    /**
-     * @deprecated Since 2.1 Use {@link #createParser(File)} instead
-     * @since 2.1
-     */
-    @Deprecated
-    @Override
-    public SmileParser createJsonParser(File f)
-        throws IOException, JsonParseException
-    {
-        return _createParser(new FileInputStream(f), _createContext(f, true));
-    }
-
-    /**
-     * @deprecated Since 2.1 Use {@link #createParser(URL)} instead
-     * @since 2.1
-     */
-    @Deprecated
-    @Override
-    public SmileParser createJsonParser(URL url)
-        throws IOException, JsonParseException
-    {
-        return _createParser(_optimizedStreamFromURL(url), _createContext(url, true));
-    }
-
-    /**
-     * @deprecated Since 2.1 Use {@link #createParser(InputStream)} instead
-     * @since 2.1
-     */
-    @Deprecated
-    @Override
-    public SmileParser createJsonParser(InputStream in)
-        throws IOException, JsonParseException
-    {
-        return _createParser(in, _createContext(in, false));
-    }
-
-    //public JsonParser createJsonParser(Reader r)
-    
-    /**
-     * @deprecated Since 2.1 Use {@link #createParser(byte[])} instead
-     * @since 2.1
-     */
-    @Deprecated
-    @Override
-    public SmileParser createJsonParser(byte[] data)
-        throws IOException, JsonParseException
-    {
-        IOContext ctxt = _createContext(data, true);
-        return _createParser(data, 0, data.length, ctxt);
-    }
-    
-    /**
-     * @deprecated Since 2.1 Use {@link #createParser(byte[],int,int)} instead
-     * @since 2.1
-     */
-    @Deprecated
-    @Override
-    public SmileParser createJsonParser(byte[] data, int offset, int len)
-        throws IOException, JsonParseException
-    {
-        return _createParser(data, offset, len, _createContext(data, true));
+        if (_inputDecorator != null) {
+            InputStream in = _inputDecorator.decorate(ctxt, data, 0, data.length);
+            if (in != null) {
+                return _createParser(_decorate(in, ctxt), ctxt);
+            }
+        }
+        return _createParser(data, offset, len, ctxt);
     }
 
     /*
     /**********************************************************
-    /* Overridden generator factory methods, new (2.1)
+    /* Overridden generator factory methods: mostly
+    /* overridden for co-variance (returns SmileGenerator)
     /**********************************************************
      */
 
@@ -389,11 +332,10 @@ public class SmileFactory extends JsonFactory
      * argument is ignored.
      */
     @Override
-    public SmileGenerator createGenerator(OutputStream out, JsonEncoding enc)
-        throws IOException
-    {
+    public SmileGenerator createGenerator(OutputStream out, JsonEncoding enc) throws IOException {
         // false -> we won't manage the stream unless explicitly directed to
-        return _createGenerator(out, _createContext(out, false));
+        IOContext ctxt = _createContext(out, false);
+        return _createGenerator(_decorate(out, ctxt), ctxt);
     }
 
     /**
@@ -404,94 +346,52 @@ public class SmileFactory extends JsonFactory
      * to be passed to this method.
      */
     @Override
-    public SmileGenerator createGenerator(OutputStream out) throws IOException
-    {
-        // false -> we won't manage the stream unless explicitly directed to
-        return _createGenerator(out, _createContext(out, false));
-    }
-    
-    /*
-    /**********************************************************
-    /* Overridden generator factory methods, old (pre-2.1)
-    /**********************************************************
-     */
-    
-    /**
-     * @deprecated Since 2.1 Use {@link #createGenerator(OutputStream)} instead
-     * @since 2.1
-     */
-    @Deprecated
-    @Override
-    public SmileGenerator createJsonGenerator(OutputStream out, JsonEncoding enc)
-        throws IOException
-    {
-        // false -> we won't manage the stream unless explicitly directed to
-        return _createGenerator(out, _createContext(out, false));
-    }
-
-    /**
-     * @deprecated Since 2.1 Use {@link #createGenerator(OutputStream)} instead
-     * @since 2.1
-     */
-    @Deprecated
-    @Override
-    public SmileGenerator createJsonGenerator(OutputStream out) throws IOException
-    {
+    public SmileGenerator createGenerator(OutputStream out) throws IOException {
         // false -> we won't manage the stream unless explicitly directed to
         IOContext ctxt = _createContext(out, false);
-        return _createGenerator(out, ctxt);
+        return _createGenerator(_decorate(out, ctxt), ctxt);
     }
 
-    @Deprecated
-    @Override
-    protected SmileGenerator _createUTF8JsonGenerator(OutputStream out, IOContext ctxt)
-        throws IOException
-    {
-        return _createGenerator(out, ctxt);
-    }
-    
     /*
     /******************************************************
     /* Overridden internal factory methods
     /******************************************************
      */
 
-    //protected IOContext _createContext(Object srcRef, boolean resourceManaged)
-
     /**
      * Overridable factory method that actually instantiates desired
      * parser.
      */
     @Override
-    protected SmileParser _createParser(InputStream in, IOContext ctxt)
-        throws IOException, JsonParseException
+    protected SmileParser _createParser(InputStream in, IOContext ctxt) throws IOException
     {
-        return new SmileParserBootstrapper(ctxt, in).constructParser(_parserFeatures,
+    	SmileParserBootstrapper bs = new SmileParserBootstrapper(ctxt, in);
+    	return bs.constructParser(_parserFeatures,
         		_smileParserFeatures, isEnabled(JsonFactory.Feature.INTERN_FIELD_NAMES),
         		_objectCodec, _rootByteSymbols);
     }
 
-    /**
-     * Overridable factory method that actually instantiates desired
-     * parser.
-     */
     @Override
-    protected JsonParser _createParser(Reader r, IOContext ctxt)
-        throws IOException, JsonParseException
+    protected JsonParser _createParser(Reader r, IOContext ctxt) throws IOException
     {
         if (_cfgDelegateToTextual) {
             return super._createParser(r, ctxt);
         }
-        throw new UnsupportedOperationException("Can not create generator for non-byte-based target");
+        return _nonByteSource();
     }
 
-    /**
-     * Overridable factory method that actually instantiates desired
-     * parser.
-     */
     @Override
-    protected SmileParser _createParser(byte[] data, int offset, int len, IOContext ctxt)
-        throws IOException, JsonParseException
+    protected JsonParser _createParser(char[] data, int offset, int len, IOContext ctxt,
+            boolean recyclable) throws IOException
+    {
+        if (_cfgDelegateToTextual) {
+            return super._createParser(data, offset, len, ctxt, recyclable);
+        }
+        return _nonByteSource();
+    }
+
+    @Override
+    protected SmileParser _createParser(byte[] data, int offset, int len, IOContext ctxt) throws IOException
     {
         return new SmileParserBootstrapper(ctxt, data, offset, len).constructParser(
                 _parserFeatures, _smileParserFeatures,
@@ -499,18 +399,13 @@ public class SmileFactory extends JsonFactory
                 _objectCodec, _rootByteSymbols);
     }
 
-    /**
-     * Overridable factory method that actually instantiates desired
-     * generator.
-     */
     @Override
-    protected JsonGenerator _createGenerator(Writer out, IOContext ctxt)
-        throws IOException
+    protected JsonGenerator _createGenerator(Writer out, IOContext ctxt) throws IOException
     {
         if (_cfgDelegateToTextual) {
             return super._createGenerator(out, ctxt);
         }
-        throw new UnsupportedOperationException("Can not create generator for non-byte-based target");
+        return _nonByteTarget(); 
     }
 
     @Override
@@ -526,17 +421,24 @@ public class SmileFactory extends JsonFactory
         if (_cfgDelegateToTextual) {
             return super._createWriter(out, enc, ctxt);
         }
-        throw new UnsupportedOperationException("Can not create generator for non-byte-based target");
+        return _nonByteTarget(); 
     }
-    
+
     /*
     /**********************************************************
     /* Internal methods
     /**********************************************************
      */
+
+    protected <T> T _nonByteSource() throws IOException {
+        throw new UnsupportedOperationException("Can not create generator for character-based (not byte-based) source");
+    }
+
+    protected <T> T _nonByteTarget() throws IOException {
+        throw new UnsupportedOperationException("Can not create generator for character-based (not byte-based) target");
+    }
     
-    protected SmileGenerator _createGenerator(OutputStream out, IOContext ctxt)
-        throws IOException
+    protected SmileGenerator _createGenerator(OutputStream out, IOContext ctxt) throws IOException
     {
         int feats = _smileGeneratorFeatures;
         /* One sanity check: MUST write header if shared string values setting is enabled,
