@@ -23,8 +23,12 @@ public class SmileGenerator
 {
     /**
      * Enumeration that defines all togglable features for Smile generators.
+     *<P>
+     * NOTE: Alas, can not implement <code>com.fasterxml.jackson.databind.cfg.ConfigFeature</code>
+     * since we do not depend on jackson-databind
      */
-    public enum Feature {
+    public enum Feature /*implements com.fasterxml.jackson.databind.cfg.ConfigFeature*/ {
+
         /**
          * Whether to write 4-byte header sequence when starting output or not.
          * If disabled, no header is written; this may be useful in embedded cases
@@ -105,9 +109,14 @@ public class SmileGenerator
             _defaultState = defaultState;
             _mask = (1 << ordinal());
         }
-        
+
+        //@Override
         public boolean enabledByDefault() { return _defaultState; }
+
+        //@Override
         public int getMask() { return _mask; }
+
+        public boolean enabledIn(int flags) { return (flags & getMask()) != 0; }    
     }
 
     /**
@@ -231,7 +240,7 @@ public class SmileGenerator
      * write can be output using back reference or not.
      */
     protected SharedStringNode[] _seenNames;
-    
+
     /**
      * Number of entries in {@link #_seenNames}; -1 if no shared name
      * detection is enabled
@@ -243,7 +252,7 @@ public class SmileGenerator
      * write can be output using back reference or not.
      */
     protected SharedStringNode[] _seenStringValues;
-    
+
     /**
      * Number of entries in {@link #_seenStringValues}; -1 if no shared text value
      * detection is enabled
@@ -403,7 +412,18 @@ public class SmileGenerator
     public Version version() {
         return PackageVersion.VERSION;
     }
-    
+
+    /*
+    /**********************************************************
+    /* Capability introspection
+    /**********************************************************
+     */
+
+    @Override
+    public boolean canWriteBinaryNatively() {
+        return true;
+    }
+
     /*
     /**********************************************************
     /* Overridden methods, configuration
@@ -518,7 +538,10 @@ public class SmileGenerator
      */
     public void writeRaw(byte b) throws IOException, JsonGenerationException
     {
-        _writeByte(TOKEN_LITERAL_START_ARRAY);
+        /* 08-Jan-2014, tatu: Should we just rather throw an exception? For now,
+         *   allow... maybe have a feature to cause an exception.
+         */
+        _writeByte(b);
     }
 
     /**
@@ -1416,14 +1439,63 @@ public class SmileGenerator
     }
 
     @Override
-    public void writeNumber(String encodedValue) throws IOException,JsonGenerationException, UnsupportedOperationException
+    public void writeNumber(String encodedValue) throws IOException
     {
-        /* 17-Apr-2010, tatu: Could try parsing etc; but for now let's not bother, it could
-         *   just be some non-standard representation that caller wants to pass
-         */
-        throw _notSupported();
+        if (encodedValue == null) {
+            writeNull();
+            return;
+        }
+        
+        // 28-May-2014, tatu: Let's actually try to support this method; should be doable
+        final int len = encodedValue.length();
+        boolean neg = encodedValue.startsWith("-");
+
+        // Let's see if it's integral or not
+        int i = neg ? 1 : 0;
+        while (true) {
+            char c = encodedValue.charAt(i);
+            if (c > '9' || c < '0') {
+                break;
+            }
+            if (++i == len) {
+                _writeIntegralNumber(encodedValue, neg);
+                return;
+            }
+        }
+        _writeDecimalNumber(encodedValue);
     }
 
+    protected void _writeIntegralNumber(String enc, boolean neg) throws IOException
+    {
+        int len = enc.length();
+        if (neg) {
+            --len;
+        }
+        // let's do approximate optimization
+        try {
+            if (len <= 9) {
+                writeNumber(Integer.parseInt(enc));
+            } else if (len <= 18) {
+                writeNumber(Long.parseLong(enc));
+            } else {
+                writeNumber(new BigInteger(enc));
+            }
+        } catch (NumberFormatException e) {
+            throw new JsonGenerationException("Invalid String representation for Number ('"+enc
+                    +"'); can not write using Smile format");
+        }
+    }
+
+    protected void _writeDecimalNumber(String enc) throws IOException
+    {
+        try {
+            writeNumber(new BigDecimal(enc));
+        } catch (NumberFormatException e) {
+            throw new JsonGenerationException("Invalid String representation for Number ('"+enc
+                    +"'); can not write using Smile format");
+        }
+    }
+    
     /*
     /**********************************************************
     /* Implementations for other methods
@@ -1458,9 +1530,6 @@ public class SmileGenerator
     @Override
     public void close() throws IOException
     {
-        /* 05-Dec-2008, tatu: To add [JACKSON-27], need to close open
-         *   scopes.
-         */
         // First: let's see that we still have buffers...
         if (_outputBuffer != null
             && isEnabled(JsonGenerator.Feature.AUTO_CLOSE_JSON_CONTENT)) {
@@ -1831,7 +1900,7 @@ public class SmileGenerator
     {
         while (bytesLeft > 0) {
             int room = _outputEnd - _outputTail;
-            if (room < 0) {
+            if (room <= 0) {
                 _flushBuffer();
                 room = _outputEnd - _outputTail;
             }
@@ -2219,9 +2288,12 @@ public class SmileGenerator
             }
         }
         // other than that, just slap it there
-        int ix = name.hashCode() & (_seenNames.length-1);
-        _seenNames[ix] = new SharedStringNode(name, _seenNameCount, _seenNames[ix]);
-        ++_seenNameCount;
+        int ref = _seenNameCount;
+        if (_validBackRef(ref)) {
+            int ix = name.hashCode() & (_seenNames.length-1);
+            _seenNames[ix] = new SharedStringNode(name, ref, _seenNames[ix]);
+        }
+        _seenNameCount = ref+1;
     }
 
     private final int _findSeenStringValue(String text)
@@ -2273,9 +2345,28 @@ public class SmileGenerator
             }
         }
         // other than that, just slap it there
-        int ix = text.hashCode() & (_seenStringValues.length-1);
-        _seenStringValues[ix] = new SharedStringNode(text, _seenStringValueCount, _seenStringValues[ix]);
-        ++_seenStringValueCount;
+        /* [Issue#18]: Except need to avoid producing bytes 0xFE and 0xFF in content;
+         *  so skip additions of those; this may produce duplicate values (and lower
+         *  efficiency), but it must be done to since these bytes must be avoided by
+         *  encoder, as per specification (except for native byte content, or as explicit
+         *  end markers). Avoiding nulls is sort of
+         */
+        int ref = _seenStringValueCount;
+        if (_validBackRef(ref)) {
+            int ix = text.hashCode() & (_seenStringValues.length-1);
+            _seenStringValues[ix] = new SharedStringNode(text, ref, _seenStringValues[ix]);
+        }
+        _seenStringValueCount = ref+1;
+    }
+
+    /**
+     * Helper method used to ensure that we do not use back-reference values
+     * that would produce illegal byte sequences (ones with byte 0xFE or 0xFF).
+     * Note that we do not try to avoid null byte (0x00) by default, although
+     * it would be technically possible as well.
+     */
+    private final static boolean _validBackRef(int index) {
+        return (index & 0xFF) < 0xFE;
     }
     
     /*
